==> v0.2.0.tar.gz: zombie-navigator/src/api/zombie/async.js <==

'use strict';


/**
	Zombie Navigator
	Copyright © 2015  Zombie Navigator Developers
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU Affero General Public License as
	published by the Free Software Foundation, either version 3 of the
	License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Affero General Public License for more details.
	
	You should have received a copy of the GNU Affero General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	
	The latest version of the GNU AGPL should be available here:
	<https://www.gnu.org/licenses/agpl.html>
	
	
	@file
	@module
	@license AGPL-3.0+
*/


/**
	ES6 implementation for ES7 async functions
	@public
	@param {function} genFunction - generator (coroutine)
*/
const run = function run (genFunction) {
	if ('function' != typeof genFunction) {
		throw new TypeError('Not a function');
	}
	
	let genObject = genFunction();
	/*if ('function' != typeof genObject[Symbol.iterator]) {
		throw new TypeError('Not a Iterable');
	}*/
	if ('function' != typeof genObject.next) {
		throw new TypeError('Not a Iterator');
	}
	
	(function exec (promiseResult, isError) {
		let item = isError
			? genObject.throw(promiseResult)
			: genObject.next(promiseResult);
		
		let throwException = e => {
			if ('function' == typeof genObject.throw) {
				exec(e, true); // throw in the generator
			} else {
				// not a generator
				// throw globally?
				throw e; // XXX: Is this appropriate?
			}
		};
		
		if (!item.done) {
			if (!(item.value instanceof Promise)) {
				throwException(new TypeError('Not a Promise'));
			}
			
			item.value
			.then(result => exec(result))
			.catch(throwException);
		}
	}());
};


/**
	Asynchronous 'sleep'
	@public
	@param {number} s - number of seconds to sleep for
	@returns {Promise}
*/
const delay = (s) => {
	return new Promise(function (resolve, reject) {
		const {setTimeout} = require('sdk/timers');
		//console.log('timer set: '+new Date);
		setTimeout(function () {
			//console.log('timer fired: '+new Date);
			resolve(true);
		}, s * 1000);
	});
};


/*
	Exports
*/
exports.run = run;
exports.delay = delay;


// vim: ts=4 noet ai

==> v0.3.0.tar.gz: zombie-navigator/src/api/zombie/async.js <==

'use strict';


/**
	Zombie Navigator
	Copyright © 2015  Zombie Navigator Developers
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU Affero General Public License as
	published by the Free Software Foundation, either version 3 of the
	License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Affero General Public License for more details.
	
	You should have received a copy of the GNU Affero General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	
	The latest version of the GNU AGPL should be available here:
	<https://www.gnu.org/licenses/agpl.html>
	
	
	@file
	@module
	@license AGPL-3.0+
*/


/**
	ES6 implementation for ES7 async functions
	@public
	@param {function} genFunction - generator (coroutine)
	@returns {Promise}
	@throws {TypeError}
*/
const run = function run (genFunction) {
	if ('function' != typeof genFunction) {
		throw new TypeError('Not a function');
	}
	
	let genObject = genFunction();
	
	if ('function' != typeof genObject.next) {
		throw new TypeError('Not a Iterator');
	}
	
	return new Promise(function (resolve, reject) {
		(function exec (promiseResult, isError) {
			try {
				let item;
				
				if (isError) {
					if ('function' != typeof genObject.throw) {
						// not a generator
						// throw globally?
						reject(e);
						
						throw e; // stop execution
					} else {
						// throw in the generator
						item = genObject.throw(promiseResult);
					}
				} else {
					item = genObject.next(promiseResult);
				}
				
				let throwException = e => {
					exec(e, true);
				};
				
				if (!item.done) {
					if (!(item.value instanceof Promise)) {
						throwException(new TypeError('Not a Promise'));
					} else {
						item.value
						.then(result => exec(result))
						.catch(throwException);
					}
				} else {
					// generator is completed
					resolve(item.value);
				}
			} catch (e) {
				reject(e);
			}
		}());
	});
};


/**
	Asynchronous 'sleep'
	@public
	@param {number} s - number of seconds to sleep for
	@returns {Promise}
*/
const delay = (s) => {
	return new Promise(function (resolve, reject) {
		const {setTimeout} = require('sdk/timers');
		//console.log('timer set: '+new Date);
		setTimeout(function () {
			//console.log('timer fired: '+new Date);
			resolve(true);
		}, s * 1000);
	});
};


/*
	Exports
*/
exports.run = run;
exports.delay = delay;


// vim: ts=4 noet ai
